module completions {

  # Fast lsp implementation for typst.
  export extern tinymist [
    --help(-h)                # Print help
    --version(-V)             # Print version
  ]

  # Probes existence (Nop run)
  export extern "tinymist probe" [
    --help(-h)                # Print help
  ]

  def "nu-complete tinymist completion shell" [] {
    [ "bash" "elvish" "fig" "fish" "powershell" "zsh" "nushell" ]
  }

  # Generates completion script to stdout
  export extern "tinymist completion" [
    shell?: string@"nu-complete tinymist completion shell" # The shell to generate the completion script for. If not provided, it will be inferred from the environment
    --help(-h)                # Print help
  ]

  # Runs language server
  export extern "tinymist lsp" [
    --mirror: string          # Mirror the stdin to the file
    --replay: string          # Replay input from the file
    --font-path: path         # Font paths
    --ignore-system-fonts     # Ensures system fonts won't be searched, unless explicitly included via `--font-path`
    --help(-h)                # Print help
  ]

  # Runs language server for tracing some typst program
  export extern "tinymist trace-lsp" [
    --persist
    --rpc-kind: string
    --mirror: string          # Mirror the stdin to the file
    --replay: string          # Replay input from the file
    input?: string            # Path to input Typst file
    --root: path              # Configures the project root (for absolute paths)
    --input: string           # Add a string key-value pair visible through `sys.inputs`
    --font-path: path         # Font paths
    --ignore-system-fonts     # Ensures system fonts won't be searched, unless explicitly included via `--font-path`
    --package-path: path      # Custom path to local packages, defaults to system-dependent location
    --package-cache-path: path # Custom path to package cache, defaults to system-dependent location
    --creation-timestamp: string # The document's creation date formatted as a UNIX timestamp (in seconds)
    --cert: path              # Path to CA certificate file for network access, especially for downloading typst packages
    --help(-h)                # Print help
  ]

  def "nu-complete tinymist preview refresh_style" [] {
    [ "onSave" "onType" ]
  }

  def "nu-complete tinymist preview preview_mode" [] {
    [ "document" "slide" ]
  }

  # Runs preview server
  export extern "tinymist preview" [
    --partial-rendering       # Only render visible part of the document. This can improve performance but still being experimental
    --invert-colors: string   # Invert colors of the preview (useful for dark themes without cost). Please note you could see the origin colors when you hover elements in the preview
    --task-id: string         # Used by lsp for identifying the task
    --refresh-style: string@"nu-complete tinymist preview refresh_style" # Used by lsp for controlling the preview refresh style
    input?: string            # Path to input Typst file
    --root: path              # Configures the project root (for absolute paths)
    --input: string           # Add a string key-value pair visible through `sys.inputs`
    --font-path: path         # Font paths
    --ignore-system-fonts     # Ensures system fonts won't be searched, unless explicitly included via `--font-path`
    --package-path: path      # Custom path to local packages, defaults to system-dependent location
    --package-cache-path: path # Custom path to package cache, defaults to system-dependent location
    --creation-timestamp: string # The document's creation date formatted as a UNIX timestamp (in seconds)
    --cert: path              # Path to CA certificate file for network access, especially for downloading typst packages
    --preview-mode: string@"nu-complete tinymist preview preview_mode" # Preview mode
    --data-plane-host: string # Data plane server will bind to this address. Note: if it equals to `static_file_host`, same address will be used
    --control-plane-host: string # Control plane server will bind to this address
    --host: string            # (File) Host for the preview server. Note: if it equals to `data_plane_host`, same address will be used
    --not-primary             # Let it not be the primary instance
    --no-open                 # Don't open the preview in the browser after compilation
    --help(-h)                # Print help (see more with '--help')
  ]

  def "nu-complete tinymist compile when" [] {
    [ "never" "onSave" "onType" "onDocumentHasTitle" ]
  }

  def "nu-complete tinymist compile format" [] {
    [ "pdf" "png" "svg" "html" ]
  }

  def "nu-complete tinymist compile pdf_standard" [] {
    [ "1.7" "a-2b" ]
  }

  # Runs compile command like `typst-cli compile`
  export extern "tinymist compile" [
    --name: string            # Give a name to the document
    input: path               # Path to input Typst file
    --root: string            # Configures the project root (for absolute paths)
    --font-path: path         # Font paths
    --ignore-system-fonts     # Ensures system fonts won't be searched, unless explicitly included via `--font-path`
    --package-path: path      # Custom path to local packages, defaults to system-dependent location
    --package-cache-path: path # Custom path to package cache, defaults to system-dependent location
    --task: string            # Name a task
    --when: string@"nu-complete tinymist compile when" # When to run the task
    output?: path             # Path to output file (PDF, PNG, SVG, or HTML). Use `-` to write output to stdout
    --format(-f): string@"nu-complete tinymist compile format" # The format of the output file, inferred from the extension by default
    --pages: string           # Which pages to export. When unspecified, all pages are exported
    --pdf-standard: string@"nu-complete tinymist compile pdf_standard" # One (or multiple comma-separated) PDF standards that Typst will enforce conformance with
    --ppi: string             # The PPI (pixels per inch) to use for PNG export
    --save-lock               # Saves the compilation arguments to the lock file
    --lockfile: path          # Specifies the path to the lock file. If the path is set, the lock file will be saved
    --help(-h)                # Print help (see more with '--help')
  ]

  def "nu-complete tinymist generate-script shell" [] {
    [ "bash" "elvish" "fish" "powershell" "zsh" ]
  }

  # Generates build script for compilation
  export extern "tinymist generate-script" [
    shell?: string@"nu-complete tinymist generate-script shell" # The shell to generate the completion script for. If not provided, it will be inferred from the environment
    --output(-o): string      # The path to the output script
    --help(-h)                # Print help
  ]

  # Runs language query
  export extern "tinymist query" [
    --help(-h)                # Print help
  ]

  # Get the documentation for a specific package
  export extern "tinymist query packageDocs" [
    --path: string            # The path of the package to request docs for
    --id: string              # The package of the package to request docs for
    --output(-o): string      # The output path for the requested docs
    --help(-h)                # Print help
  ]

  # Check a specific package
  export extern "tinymist query checkPackage" [
    --path: string            # The path of the package to request docs for
    --id: string              # The package of the package to request docs for
    --output(-o): string      # The output path for the requested docs
    --help(-h)                # Print help
  ]

  # Print this message or the help of the given subcommand(s)
  export extern "tinymist query help" [
  ]

  # Get the documentation for a specific package
  export extern "tinymist query help packageDocs" [
  ]

  # Check a specific package
  export extern "tinymist query help checkPackage" [
  ]

  # Print this message or the help of the given subcommand(s)
  export extern "tinymist query help help" [
  ]

  # Runs documents
  export extern "tinymist doc" [
    --help(-h)                # Print help
  ]

  # Declare a document (project input)
  export extern "tinymist doc new" [
    --name: string            # Give a name to the document
    input: path               # Path to input Typst file
    --root: string            # Configures the project root (for absolute paths)
    --font-path: path         # Font paths
    --ignore-system-fonts     # Ensures system fonts won't be searched, unless explicitly included via `--font-path`
    --package-path: path      # Custom path to local packages, defaults to system-dependent location
    --package-cache-path: path # Custom path to package cache, defaults to system-dependent location
    --help(-h)                # Print help
  ]

  # Configure document priority in workspace
  export extern "tinymist doc configure" [
    --name: string            # Give a name to the document
    input: path               # Path to input Typst file
    --priority: string        # Set the unsigned priority of these task (lower numbers are higher priority)
    --help(-h)                # Print help
  ]

  # Print this message or the help of the given subcommand(s)
  export extern "tinymist doc help" [
  ]

  # Declare a document (project input)
  export extern "tinymist doc help new" [
  ]

  # Configure document priority in workspace
  export extern "tinymist doc help configure" [
  ]

  # Print this message or the help of the given subcommand(s)
  export extern "tinymist doc help help" [
  ]

  # Runs tasks
  export extern "tinymist task" [
    --help(-h)                # Print help
  ]

  def "nu-complete tinymist task preview when" [] {
    [ "never" "onSave" "onType" "onDocumentHasTitle" ]
  }

  def "nu-complete tinymist task preview refresh_style" [] {
    [ "onSave" "onType" ]
  }

  def "nu-complete tinymist task preview preview_mode" [] {
    [ "document" "slide" ]
  }

  # Declare a preview task
  export extern "tinymist task preview" [
    --name: string            # Give a name to the document
    input: path               # Path to input Typst file
    --root: string            # Configures the project root (for absolute paths)
    --font-path: path         # Font paths
    --ignore-system-fonts     # Ensures system fonts won't be searched, unless explicitly included via `--font-path`
    --package-path: path      # Custom path to local packages, defaults to system-dependent location
    --package-cache-path: path # Custom path to package cache, defaults to system-dependent location
    --task: string            # Name a task
    --when: string@"nu-complete tinymist task preview when" # When to run the task
    --partial-rendering       # Only render visible part of the document. This can improve performance but still being experimental
    --invert-colors: string   # Invert colors of the preview (useful for dark themes without cost). Please note you could see the origin colors when you hover elements in the preview
    --task-id: string         # Used by lsp for identifying the task
    --refresh-style: string@"nu-complete tinymist task preview refresh_style" # Used by lsp for controlling the preview refresh style
    --preview-mode: string@"nu-complete tinymist task preview preview_mode" # Preview mode
    --help(-h)                # Print help (see more with '--help')
  ]

  # Print this message or the help of the given subcommand(s)
  export extern "tinymist task help" [
  ]

  # Declare a preview task
  export extern "tinymist task help preview" [
  ]

  # Print this message or the help of the given subcommand(s)
  export extern "tinymist task help help" [
  ]

  # Print this message or the help of the given subcommand(s)
  export extern "tinymist help" [
  ]

  # Probes existence (Nop run)
  export extern "tinymist help probe" [
  ]

  # Generates completion script to stdout
  export extern "tinymist help completion" [
  ]

  # Runs language server
  export extern "tinymist help lsp" [
  ]

  # Runs language server for tracing some typst program
  export extern "tinymist help trace-lsp" [
  ]

  # Runs preview server
  export extern "tinymist help preview" [
  ]

  # Runs compile command like `typst-cli compile`
  export extern "tinymist help compile" [
  ]

  # Generates build script for compilation
  export extern "tinymist help generate-script" [
  ]

  # Runs language query
  export extern "tinymist help query" [
  ]

  # Get the documentation for a specific package
  export extern "tinymist help query packageDocs" [
  ]

  # Check a specific package
  export extern "tinymist help query checkPackage" [
  ]

  # Runs documents
  export extern "tinymist help doc" [
  ]

  # Declare a document (project input)
  export extern "tinymist help doc new" [
  ]

  # Configure document priority in workspace
  export extern "tinymist help doc configure" [
  ]

  # Runs tasks
  export extern "tinymist help task" [
  ]

  # Declare a preview task
  export extern "tinymist help task preview" [
  ]

  # Print this message or the help of the given subcommand(s)
  export extern "tinymist help help" [
  ]

}

export use completions *
